import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { NetTransactions } from './net-transactions';

const url = '/api/net-transactions';
const serviceName = 'NetTransactionsService';

@Injectable({
  providedIn: 'root',
})
export class NetTransactionsService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  list(startDate: string, finishDate): Observable<NetTransactions> {
    const startDateWithSlash = startDate ? `/${startDate}` : '';
    const finishDateWithSlash = finishDate ? `/${finishDate}` : '';
    return <Observable<NetTransactions>>(
      this.http
        .get<NetTransactions>(`${url}${startDateWithSlash}${finishDateWithSlash}`)
        .pipe(catchError(this.log.handleError(serviceName, 'list')))
    );
  }
}
