import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';
import { ProductGroup } from '../core/product-group';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/product-groups';
const serviceName = 'ProductGroupService';

@Injectable({
  providedIn: 'root',
})
export class ProductGroupService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string): Observable<ProductGroup> {
    const getUrl: string = id === null ? `${url}` : `${url}/${id}`;
    return <Observable<ProductGroup>>(
      this.http
        .get<ProductGroup>(getUrl)
        .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`)))
    );
  }

  list(): Observable<ProductGroup[]> {
    return <Observable<ProductGroup[]>>(
      this.http
        .get<ProductGroup[]>(`${url}/list`)
        .pipe(catchError(this.log.handleError(serviceName, 'list')))
    );
  }

  save(productGroup: ProductGroup): Observable<ProductGroup> {
    return <Observable<ProductGroup>>(
      this.http
        .post<ProductGroup>(`${url}`, productGroup, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'save')))
    );
  }

  update(productGroup: ProductGroup): Observable<ProductGroup> {
    return <Observable<ProductGroup>>(
      this.http
        .put<ProductGroup>(`${url}/${productGroup.id}`, productGroup, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'update')))
    );
  }

  saveOrUpdate(productGroup: ProductGroup): Observable<ProductGroup> {
    if (!productGroup.id) {
      return this.save(productGroup);
    }
    return this.update(productGroup);
  }

  delete(id: string): Observable<ProductGroup> {
    return <Observable<ProductGroup>>(
      this.http
        .delete<ProductGroup>(`${url}/${id}`, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'delete')))
    );
  }
}
