import { Component, Inject, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { MatAutocompleteSelectedEvent } from '@angular/material/autocomplete';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { Observable } from 'rxjs';
import { debounceTime, distinctUntilChanged, startWith, switchMap } from 'rxjs/operators';

import { Batch } from '../core/batch';
import { BatchService } from '../core/batch.service';
import { MathService } from '../shared/math.service';

@Component({
  selector: 'app-purchase-return-dialog',
  templateUrl: './purchase-return-dialog.component.html',
  styleUrls: ['./purchase-return-dialog.component.css'],
})
export class PurchaseReturnDialogComponent implements OnInit {
  batches: Observable<Batch[]>;
  form: FormGroup;
  batch: Batch;

  constructor(
    public dialogRef: MatDialogRef<PurchaseReturnDialogComponent>,
    @Inject(MAT_DIALOG_DATA) public data: any,
    private fb: FormBuilder,
    private math: MathService,
    private batchSer: BatchService,
  ) {
    this.createForm();
    this.listenToBatchAutocompleteChange();
  }

  ngOnInit() {
    this.form.setValue({
      batch: this.data.inventory.batch,
      quantity: this.data.inventory.quantity,
    });
    this.batch = this.data.inventory.batch;
  }

  createForm() {
    this.form = this.fb.group({
      batch: '',
      quantity: '',
    });
  }

  listenToBatchAutocompleteChange(): void {
    const control = this.form.get('batch');
    this.batches = control.valueChanges.pipe(
      startWith('null'),
      debounceTime(150),
      distinctUntilChanged(),
      switchMap((x) => this.batchSer.autocomplete(this.data.date, x)),
    );
  }

  displayFn(batch?: Batch): string | undefined {
    return batch ? batch.name : undefined;
  }

  batchSelected(event: MatAutocompleteSelectedEvent): void {
    this.batch = event.option.value;
  }

  accept(): void {
    const formValue = this.form.value;
    const quantity = this.math.parseAmount(formValue.quantity, 2);
    this.data.inventory.batch = this.batch;
    this.data.inventory.product = this.batch.product;
    this.data.inventory.quantity = quantity;
    this.data.inventory.rate = this.batch.rate;
    this.data.inventory.tax = this.batch.tax;
    this.data.inventory.discount = this.batch.discount;
    this.data.inventory.amount =
      quantity * this.batch.rate * (1 + this.batch.tax) * (1 - this.batch.discount);
    this.dialogRef.close(this.data.inventory);
  }
}
