import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';
import { Product } from '../core/product';

const serviceName = 'SettingsService';

@Injectable({ providedIn: 'root' })
export class SettingsService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  getLockInformation(): Observable<any> {
    const url = '/api/lock-information';
    return <Observable<any>>(
      this.http
        .get<any>(url)
        .pipe(catchError(this.log.handleError(serviceName, 'getLockInformation')))
    );
  }

  setLockInformation(lockInformation: any): Observable<any> {
    const url = '/api/lock-information';
    return <Observable<any>>(
      this.http
        .post<any>(url, lockInformation)
        .pipe(catchError(this.log.handleError(serviceName, 'setLockInformation')))
    );
  }

  deleteLockInformation(): Observable<any> {
    const url = '/api/lock-information';
    return <Observable<any>>(
      this.http
        .delete<any>(url)
        .pipe(catchError(this.log.handleError(serviceName, 'deleteLockInformation')))
    );
  }

  resetStock(
    resetDate: string,
    stockDate: string,
    product: Product,
    quantity: number,
  ): Observable<boolean> {
    const url = '/api/reset-stock';
    return this.http
      .post<any>(`${url}/${product.id}`, { stockDate, resetDate, quantity })
      .pipe(catchError(this.log.handleError(serviceName, 'resetStock')));
  }

  rebaseDatabase(date: string): Observable<boolean> {
    const url = '/api/rebase';
    return <Observable<any>>(
      this.http
        .post<any>(`${url}/${date}`, {})
        .pipe(catchError(this.log.handleError(serviceName, 'rebaseDatabase')))
    );
  }

  setMaintenance(enable: boolean): Observable<any> {
    const url = '/api/maintenance';
    return this.http
      .post<any>(url, { enabled: enable })
      .pipe(catchError(this.log.handleError(serviceName, 'setMaintenance')));
  }

  getMaintenance(): Observable<any> {
    const url = '/api/maintenance';
    return <Observable<any>>(
      this.http.get<any>(url).pipe(catchError(this.log.handleError(serviceName, 'getMaintenance')))
    );
  }

  checkDatabaseIntegrity(): Observable<any> {
    const url = '/api/db-integrity';
    return <Observable<any>>(
      this.http
        .post<any>(url, {})
        .pipe(catchError(this.log.handleError(serviceName, 'checkDatabaseIntegrity')))
    );
  }
}
