import { Injectable } from '@angular/core';
import { fromEvent, of, zip } from 'rxjs';
import { map } from 'rxjs/operators';

import { Voucher } from '../core/voucher';

@Injectable({
  providedIn: 'root',
})
export class ImageService {
  // eslint-disable-next-line class-methods-use-this
  resizeImage(image, MAX_WIDTH, MAX_HEIGHT) {
    const canvas = document.createElement('canvas');
    const ctx = canvas.getContext('2d');
    const img = new Image();
    const ex = fromEvent(img, 'load').pipe(
      map(() => {
        let width = img.naturalWidth;
        let height = img.naturalHeight;
        const ratio = Math.min(1, MAX_WIDTH / width, MAX_HEIGHT / height);

        if (ratio === 1) {
          return image;
        }

        width *= ratio;
        height *= ratio;

        canvas.width = width;
        canvas.height = height;
        ctx.drawImage(img, 0, 0, width, height);
        return canvas.toDataURL('image/jpeg', 0.95);
      }),
    );
    img.src = image;
    return ex;
  }

  detectFiles(event, voucher: Voucher) {
    const { files } = event.target;
    if (!files) {
      return;
    }
    for (let i = 0; i < files.length; i += 1) {
      const file = files[i];
      const reader = new FileReader();
      reader.onload = (e: any) => {
        zip(
          of(e.target.result),
          this.resizeImage(e.target.result, 100, 150),
          this.resizeImage(e.target.result, 825, 1170),
        ).subscribe((val) => voucher.files.push({ id: null, thumbnail: val[1], resized: val[2] }));
      };
      reader.readAsDataURL(file);
    }
  }
}
