import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';
import { User } from '../core/user';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/users';
const serviceName = 'UserService';

@Injectable({
  providedIn: 'root',
})
export class UserService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string): Observable<User> {
    const getUrl: string = id === null ? `${url}` : `${url}/${id}`;
    return <Observable<User>>(
      this.http
        .get<User>(getUrl)
        .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`)))
    );
  }

  list(): Observable<User[]> {
    return <Observable<User[]>>(
      this.http
        .get<User[]>(`${url}/list`)
        .pipe(catchError(this.log.handleError(serviceName, 'list')))
    );
  }

  listOfNames(): Observable<string[]> {
    return <Observable<string[]>>(
      this.http
        .get<string[]>(`${url}/active`)
        .pipe(catchError(this.log.handleError(serviceName, 'list')))
    );
  }

  save(user: User): Observable<User> {
    return <Observable<User>>(
      this.http
        .post<User>(`${url}`, user, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'save')))
    );
  }

  update(user: User): Observable<User> {
    return <Observable<User>>(
      this.http
        .put<User>(`${url}/${user.id}`, user, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'update')))
    );
  }

  updateMe(user: User): Observable<User> {
    return <Observable<User>>(
      this.http
        .put<User>(`${url}/me`, user, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'update')))
    );
  }

  saveOrUpdate(user: User): Observable<User> {
    if (!user.id) {
      return this.save(user);
    }
    return this.update(user);
  }

  delete(id: string): Observable<User> {
    return <Observable<User>>(
      this.http
        .delete<User>(`${url}/${id}`, httpOptions)
        .pipe(catchError(this.log.handleError(serviceName, 'delete')))
    );
  }
}
